/**
 * @file HuffmanCoder.cpp
 * @brief Definition of the HuffmanCoder class.
 * Version: 23.02.2009
 * @author Ruediger Knoerig
 */

#include "HuffmanCoder.h"

#include <itpp/base/sort.h>
#include <list>

namespace itpp {

double HuffmanCoder::train(const Vec<unsigned int> &trainingdata)
{
	itpp::Vec<HuffmanCoder::sort_elem> liste(N);
	for(unsigned int k=0;k<N;k++) liste(k).setSymbol(k);
	unsigned int Ntr=trainingdata.length();
	for(unsigned int k=0;k<Ntr;k++) ++liste(trainingdata(k));
	itpp::sort(liste);

	itpp::bvec code(N-1);
	code.clear();
	codetable(liste(0).getSymbol())=code;
	code(N-2)=1;
	for(unsigned int k=1;k<N;k++)
	{
		codetable(liste(k).getSymbol())=code.right(N-k);
	}

	// berechne die mittlere Coderate
	double Nrez=1.0/Ntr;
	double mcl=0.0;
	for(unsigned int k=0;k<N;k++)
	{
		HuffmanCoder::sort_elem e=liste(k);
		mcl += e.getFrequency()*Nrez*codetable(e.getSymbol()).length();
		decodetable(N-k-1)=e.getSymbol();
	}
	trained=true;
	return mcl;
}

itpp::Vec<unsigned int> HuffmanCoder::decode(itpp::bvec coded)
{
	std::list<unsigned int> decv;
	unsigned int zerocounter=0;
	int k=0;
	for(;k<coded.length();k++)
	{
		if(coded._elem(k)==1)
		{
			decv.push_back(decodetable._elem(zerocounter));
			zerocounter=0;
		}
		else ++zerocounter;
		if(zerocounter == N-1)
		{
			decv.push_back(decodetable._elem(zerocounter));
			zerocounter=0;
		}
	}
	itpp::Vec<unsigned int> decoded(decv.size());
	int l=0;
	for(std::list<unsigned int>::const_iterator it=decv.begin();it != decv.end();it++)
	{
		decoded(l++)=*it;
	}
	return decoded;
}

}
